@@ -36,11 +36,8 @@ use 5.012;
 use strict;
 use warnings FATAL => 'all';
 use App::CELL qw( $CELL $log $meta $site );
-#use App::Dochazka::REST::dbh qw( $dbh );
 use App::Dochazka::REST::Model::Shared qw( load cud );
-use Carp;
 use Data::Dumper;
-use DBI;
 use Params::Validate qw( :all );
 
 # we get 'spawn', 'reset', and accessors from parent
@@ -58,11 +55,11 @@ App::Dochazka::REST::Model::Schedhistory - schedule history functions
 
 =head1 VERSION
 
-Version 0.322
+Version 0.348
 
 =cut
 
-our $VERSION = '0.322';
+our $VERSION = '0.348';
 
 
 
@@ -164,13 +161,15 @@ record is found, it will be in the payload and the code will be
 
 sub load_by_eid {
     my $self = shift;
-    my ( $eid, $ts ) = validate_pos( @_, 
+    my ( $conn, $eid, $ts ) = validate_pos( @_, 
+        { isa => 'DBIx::Connector' },
         { type => SCALAR },                # EID
         { type => SCALAR, optional => 1 }, # optional timestamp
     );
 
     if ( $ts ) {
         return load(
+            conn => $conn,
             class => __PACKAGE__,
             sql => $site->SQL_SCHEDHISTORY_SELECT_ARBITRARY,
             keys => [ $eid, $ts ],
@@ -178,6 +177,7 @@ sub load_by_eid {
     }
 
     return load(
+        conn => $conn,
         class => __PACKAGE__,
         sql => $site->SQL_SCHEDHISTORY_SELECT_CURRENT,
         keys => [ $eid ],
@@ -194,9 +194,13 @@ Given a shid, load a single schedhistory record.
 
 sub load_by_id {
     my $self = shift;
-    my ( $shid ) = validate_pos( @_, { type => SCALAR } );
+    my ( $conn, $shid ) = validate_pos( @_, 
+        { isa => 'DBIx::Connector' },
+        { type => SCALAR } 
+    );
 
     return load(
+        conn => $conn,
         class => __PACKAGE__,
         sql => $site->SQL_SCHEDHISTORY_SELECT_BY_SHID,
         keys => [ $shid ],
@@ -212,7 +216,12 @@ Wrapper for load_by_id
 
 sub load_by_shid {
     my $self = shift;
-    return $self->load_by_id( @_ );
+    my ( $conn, $shid ) = validate_pos( @_, 
+        { isa => 'DBIx::Connector' },
+        { type => SCALAR } 
+    );
+
+    return $self->load_by_id( $conn, $shid );
 }
 
 
@@ -224,9 +233,12 @@ Field values are taken from the object. Returns a status object.
 =cut
 
 sub insert {
-    my ( $self ) = @_;
+    my $self = shift;
+    my ( $context ) = validate_pos( @_, { type => HASHREF } );
 
     my $status = cud(
+        conn => $context->{'dbix_conn'},
+        eid => $context->{'current'}->{'eid'},
         object => $self,
         sql => $site->SQL_SCHEDHISTORY_INSERT,
         attrs => [ 'eid', 'sid', 'effective', 'remark' ],
@@ -249,9 +261,12 @@ Instance method. Deletes the record. Returns status object.
 =cut
 
 sub delete {
-    my ( $self ) = @_;
+    my $self = shift;
+    my ( $context ) = validate_pos( @_, { type => HASHREF } );
 
     my $status = cud(
+        conn => $context->{'dbix_conn'},
+        eid => $context->{'current'}->{'eid'},
         object => $self,
         sql => $site->SQL_SCHEDHISTORY_DELETE,
         attrs => [ 'shid' ],
@@ -282,7 +297,8 @@ the payload will be undefined.
 =cut
 
 sub get_schedhistory {
-    return App::Dochazka::REST::Model::Shared::get_history( 'sched', @_ );
+    my $context = shift;
+    return App::Dochazka::REST::Model::Shared::get_history( 'sched', $context->{'dbix_conn'}, @_ );
 }
 
 
