@@ -50,11 +50,11 @@ App::Dochazka::REST::Util - miscellaneous utilities
 
 =head1 VERSION
 
-Version 0.322
+Version 0.348
 
 =cut
 
-our $VERSION = '0.322';
+our $VERSION = '0.348';
 
 
 
@@ -63,7 +63,7 @@ our $VERSION = '0.322';
 
 Miscellaneous utilities
 
-    use App::Dochazka::REST::Util::Timestamp;
+    use App::Dochazka::REST::Util;
 
     ...
 
